@@ -1,7 +1,7 @@
 /*
  * FD polling functions for generic poll()
  *
- * Copyright 2000-2008 Willy Tarreau <w@1wt.eu>
+ * Copyright 2000-2014 Willy Tarreau <w@1wt.eu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@ static unsigned int *fd_evts[2];
 /* private data */
 static struct pollfd *poll_events = NULL;
 
+
 static inline unsigned int hap_fd_isset(int fd, unsigned int *evts)
 {
 	return evts[fd / (8*sizeof(*evts))] & (1U << (fd & (8*sizeof(*evts) - 1)));
@@ -46,47 +47,8 @@ static inline void hap_fd_clr(int fd, unsigned int *evts)
 {
 	evts[fd / (8*sizeof(*evts))] &= ~(1U << (fd & (8*sizeof(*evts) - 1)));
 }
-/*
- * Benchmarks performed on a Pentium-M notebook show that using functions
- * instead of the usual macros improve the FD_* performance by about 80%,
- * and that marking them regparm(2) adds another 20%.
- */
-REGPRM2 static int __fd_is_set(const int fd, int dir)
-{
-	return hap_fd_isset(fd, fd_evts[dir]);
-}
 
-REGPRM2 static int __fd_set(const int fd, int dir)
-{
-	hap_fd_set(fd, fd_evts[dir]);
-	return 0;
-}
-
-REGPRM2 static int __fd_clr(const int fd, int dir)
-{
-	hap_fd_clr(fd, fd_evts[dir]);
-	return 0;
-}
-
-REGPRM2 static int __fd_cond_s(const int fd, int dir)
-{
-	int ret;
-	ret = !hap_fd_isset(fd, fd_evts[dir]);
-	if (ret)
-		hap_fd_set(fd, fd_evts[dir]);
-	return ret;
-}
-
-REGPRM2 static int __fd_cond_c(const int fd, int dir)
-{
-	int ret;
-	ret = hap_fd_isset(fd, fd_evts[dir]);
-	if (ret)
-		hap_fd_clr(fd, fd_evts[dir]);
-	return ret;
-}
-
-REGPRM1 static void __fd_rem(const int fd)
+REGPRM1 static void __fd_clo(int fd)
 {
 	hap_fd_clr(fd, fd_evts[DIR_RD]);
 	hap_fd_clr(fd, fd_evts[DIR_WR]);
@@ -100,11 +62,42 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 	int status;
 	int fd, nbfd;
 	int wait_time;
-
+	int updt_idx, en, eo;
 	int fds, count;
 	int sr, sw;
 	unsigned rn, wn; /* read new, write new */
 
+	/* first, scan the update list to find changes */
+	for (updt_idx = 0; updt_idx < fd_nbupdt; updt_idx++) {
+		fd = fd_updt[updt_idx];
+		fdtab[fd].updated = 0;
+		fdtab[fd].new = 0;
+
+		if (!fdtab[fd].owner)
+			continue;
+
+		eo = fdtab[fd].state;
+		en = fd_compute_new_polled_status(eo);
+
+		if ((eo ^ en) & FD_EV_POLLED_RW) {
+			/* poll status changed, update the lists */
+			fdtab[fd].state = en;
+
+			if ((eo & ~en) & FD_EV_POLLED_R)
+				hap_fd_clr(fd, fd_evts[DIR_RD]);
+			else if ((en & ~eo) & FD_EV_POLLED_R)
+				hap_fd_set(fd, fd_evts[DIR_RD]);
+
+			if ((eo & ~en) & FD_EV_POLLED_W)
+				hap_fd_clr(fd, fd_evts[DIR_WR]);
+			else if ((en & ~eo) & FD_EV_POLLED_W)
+				hap_fd_set(fd, fd_evts[DIR_WR]);
+		}
+
+		fd_alloc_or_release_cache_entry(fd, en);
+	}
+	fd_nbupdt = 0;
+
 	nbfd = 0;
 	for (fds = 0; (fds * 8*sizeof(**fd_evts)) < maxfd; fds++) {
 		rn = fd_evts[DIR_RD][fds];
@@ -125,7 +118,7 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 	}
       
 	/* now let's wait for events */
-	if (run_queue || signal_queue_len)
+	if (fd_cache_num || run_queue || signal_queue_len)
 		wait_time = 0;
 	else if (!exp)
 		wait_time = MAX_DELAY_MS;
@@ -137,31 +130,41 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 			wait_time = MAX_DELAY_MS;
 	}
 
+	gettimeofday(&before_poll, NULL);
 	status = poll(poll_events, nbfd, wait_time);
 	tv_update_date(wait_time, status);
+	measure_idle();
 
 	for (count = 0; status > 0 && count < nbfd; count++) {
+		int e = poll_events[count].revents;
 		fd = poll_events[count].fd;
 	  
-		if (!(poll_events[count].revents & ( POLLOUT | POLLIN | POLLERR | POLLHUP )))
+		if (!(e & ( POLLOUT | POLLIN | POLLERR | POLLHUP )))
 			continue;
 
 		/* ok, we found one active fd */
 		status--;
 
-		if (hap_fd_isset(fd, fd_evts[DIR_RD])) {
-			if (fdtab[fd].state == FD_STCLOSE)
-				continue;
-			if (poll_events[count].revents & ( POLLIN | POLLERR | POLLHUP ))
-				fdtab[fd].cb[DIR_RD].f(fd);
+		if (!fdtab[fd].owner)
+			continue;
+
+		/* it looks complicated but gcc can optimize it away when constants
+		 * have same values... In fact it depends on gcc :-(
+		 */
+		fdtab[fd].ev &= FD_POLL_STICKY;
+		if (POLLIN == FD_POLL_IN && POLLOUT == FD_POLL_OUT &&
+		    POLLERR == FD_POLL_ERR && POLLHUP == FD_POLL_HUP) {
+			fdtab[fd].ev |= e & (POLLIN|POLLOUT|POLLERR|POLLHUP);
 		}
-	  
-		if (hap_fd_isset(fd, fd_evts[DIR_WR])) {
-			if (fdtab[fd].state == FD_STCLOSE)
-				continue;
-			if (poll_events[count].revents & ( POLLOUT | POLLERR | POLLHUP ))
-				fdtab[fd].cb[DIR_WR].f(fd);
+		else {
+			fdtab[fd].ev |=
+				((e & POLLIN ) ? FD_POLL_IN  : 0) |
+				((e & POLLOUT) ? FD_POLL_OUT : 0) |
+				((e & POLLERR) ? FD_POLL_ERR : 0) |
+				((e & POLLHUP) ? FD_POLL_HUP : 0);
 		}
+
+		fd_process_polled_events(fd);
 	}
 
 }
@@ -241,16 +244,11 @@ static void _do_register(void)
 	p->pref = 200;
 	p->private = NULL;
 
+	p->clo  = __fd_clo;
 	p->test = _do_test;
 	p->init = _do_init;
 	p->term = _do_term;
 	p->poll = _do_poll;
-	p->is_set = __fd_is_set;
-	p->set = __fd_set;
-	p->clr = __fd_clr;
-	p->clo = p->rem = __fd_rem;
-	p->cond_s = __fd_cond_s;
-	p->cond_c = __fd_cond_c;
 }
 
 
