@@ -765,7 +765,7 @@ void send_log(struct proxy *p, int level, const char *format, ...)
 	char *dataptr;
 	int  data_len;
 
-	if (level < 0 || format == NULL)
+	if (level < 0 || format == NULL || logline == NULL)
 		return;
 
 	dataptr = update_log_hdr(); /* update log header and skip it */
