@@ -11,6 +11,7 @@
  */
 
 #include <unistd.h>
+#include <fcntl.h>
 
 #include <common/config.h>
 #include <common/memory.h>
@@ -58,6 +59,10 @@ struct pipe *get_pipe()
 		pool_free2(pool2_pipe, ret);
 		return NULL;
 	}
+#ifdef F_SETPIPE_SZ
+	if (global.tune.pipesize)
+		fcntl(pipefd[0], F_SETPIPE_SZ, global.tune.pipesize);
+#endif
 	ret->data = 0;
 	ret->prod = pipefd[1];
 	ret->cons = pipefd[0];
