@@ -16,6 +16,8 @@
 
 #include <proto/queue.h>
 #include <proto/server.h>
+#include <proto/session.h>
+#include <proto/stream_interface.h>
 #include <proto/task.h>
 
 
@@ -48,7 +50,7 @@ unsigned int srv_dynamic_maxconn(const struct server *s)
 	else max = MAX(s->minconn,
 		       s->proxy->beconn * s->maxconn / s->proxy->fullconn);
 
-	if ((s->state & SRV_WARMINGUP) &&
+	if ((s->state == SRV_ST_STARTING) &&
 	    now.tv_sec < s->last_change + s->slowstart &&
 	    now.tv_sec >= s->last_change) {
 		unsigned int ratio;
@@ -100,30 +102,28 @@ struct session *pendconn_get_next_sess(struct server *srv, struct proxy *px)
 	struct session *sess;
 	struct server *rsrv;
 
-	rsrv = srv->tracked;
+	rsrv = srv->track;
 	if (!rsrv)
 		rsrv = srv;
 
 	ps = pendconn_from_srv(srv);
 	pp = pendconn_from_px(px);
 	/* we want to get the definitive pendconn in <ps> */
-	if (!pp || !srv_is_usable(rsrv->state, rsrv->eweight)) {
+	if (!pp || !srv_is_usable(rsrv)) {
 		if (!ps)
 			return NULL;
 	} else {
 		/* pendconn exists in the proxy queue */
-		if (!ps || tv_islt(&pp->sess->logs.tv_request, &ps->sess->logs.tv_request)) {
+		if (!ps || tv_islt(&pp->sess->logs.tv_request, &ps->sess->logs.tv_request))
 			ps = pp;
-			ps->sess->srv = srv;
-		}
 	}
 	sess = ps->sess;
 	pendconn_free(ps);
 
 	/* we want to note that the session has now been assigned a server */
 	sess->flags |= SN_ASSIGNED;
-	sess->srv = srv;
-	sess->srv_conn = srv;
+	sess->target = &srv->obj_type;
+	session_add_srv_conn(sess, srv);
 	srv->served++;
 	if (px->lbprm.server_take_conn)
 		px->lbprm.server_take_conn(srv);
@@ -140,6 +140,7 @@ struct session *pendconn_get_next_sess(struct server *srv, struct proxy *px)
 struct pendconn *pendconn_add(struct session *sess)
 {
 	struct pendconn *p;
+	struct server *srv;
 
 	p = pool_alloc2(pool2_pendconn);
 	if (!p)
@@ -147,25 +148,80 @@ struct pendconn *pendconn_add(struct session *sess)
 
 	sess->pend_pos = p;
 	p->sess = sess;
-	p->srv  = sess->srv;
+	p->srv = srv = objt_server(sess->target);
 
-	if (sess->flags & SN_ASSIGNED && sess->srv) {
-		LIST_ADDQ(&sess->srv->pendconns, &p->list);
-		sess->srv->nbpend++;
-		sess->logs.srv_queue_size += sess->srv->nbpend;
-		if (sess->srv->nbpend > sess->srv->counters.nbpend_max)
-			sess->srv->counters.nbpend_max = sess->srv->nbpend;
+	if (sess->flags & SN_ASSIGNED && srv) {
+		LIST_ADDQ(&srv->pendconns, &p->list);
+		srv->nbpend++;
+		sess->logs.srv_queue_size += srv->nbpend;
+		if (srv->nbpend > srv->counters.nbpend_max)
+			srv->counters.nbpend_max = srv->nbpend;
 	} else {
 		LIST_ADDQ(&sess->be->pendconns, &p->list);
 		sess->be->nbpend++;
 		sess->logs.prx_queue_size += sess->be->nbpend;
-		if (sess->be->nbpend > sess->be->counters.nbpend_max)
-			sess->be->counters.nbpend_max = sess->be->nbpend;
+		if (sess->be->nbpend > sess->be->be_counters.nbpend_max)
+			sess->be->be_counters.nbpend_max = sess->be->nbpend;
 	}
 	sess->be->totpend++;
 	return p;
 }
 
+/* Redistribute pending connections when a server goes down. The number of
+ * connections redistributed is returned.
+ */
+int pendconn_redistribute(struct server *s)
+{
+	struct pendconn *pc, *pc_bck;
+	int xferred = 0;
+
+	list_for_each_entry_safe(pc, pc_bck, &s->pendconns, list) {
+		struct session *sess = pc->sess;
+
+		if ((sess->be->options & (PR_O_REDISP|PR_O_PERSIST)) == PR_O_REDISP &&
+		    !(sess->flags & SN_FORCE_PRST)) {
+			/* The REDISP option was specified. We will ignore
+			 * cookie and force to balance or use the dispatcher.
+			 */
+
+			/* it's left to the dispatcher to choose a server */
+			sess->flags &= ~(SN_DIRECT | SN_ASSIGNED | SN_ADDR_SET);
+
+			pendconn_free(pc);
+			task_wakeup(sess->task, TASK_WOKEN_RES);
+			xferred++;
+		}
+	}
+	return xferred;
+}
+
+/* Check for pending connections at the backend, and assign some of them to
+ * the server coming up. The server's weight is checked before being assigned
+ * connections it may not be able to handle. The total number of transferred
+ * connections is returned.
+ */
+int pendconn_grab_from_px(struct server *s)
+{
+	int xferred;
+
+	if (!srv_is_usable(s))
+		return 0;
+
+	for (xferred = 0; !s->maxconn || xferred < srv_dynamic_maxconn(s); xferred++) {
+		struct session *sess;
+		struct pendconn *p;
+
+		p = pendconn_from_px(s->proxy);
+		if (!p)
+			break;
+		p->sess->target = &s->obj_type;
+		sess = p->sess;
+		pendconn_free(p);
+		task_wakeup(sess->task, TASK_WOKEN_RES);
+	}
+	return xferred;
+}
+
 /*
  * Detaches pending connection <p>, decreases the pending count, and frees
  * the pending connection. The connection might have been queued to a specific
